/************************************************************/
/* an example program to demonstrate a basic use of pythia8 */
/* for an LHC setup                                         */
/************************************************************/

//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------

using namespace Moses1;
using namespace Pythia8;

int main() {
	// Generator. Process selection. LHC initialization. Histogram.
	Pythia pythia;
	int absidLepton = 13;
	string absidLepton_str = "13";

	// to switch on the process ffbar--->gamma/Z
	pythia.readString("WeakSingleBoson:ffbar2gmZ = on");
	
	//pythia.readString("WeakZ0:gmZmode = 1"); // only photon propagator (can be usefull for tests)
	//pythia.readString("WeakZ0:gmZmode = 2"); // only Z0 propagator (can be usefull for tests)

	// rule out all gamma/Z decays except for muons
	//pythia.readString("22:onMode = off"); // switch off all of the gamma decay modes
	//pythia.readString("22:onIfAny = 13"); // switch on the gamma->mu-mu+ decay mode only
	pythia.readString("23:onMode = off"); // switch off all of the Z0 decay modes
	pythia.readString("23:onIfAny = " + absidLepton_str); // switch on the Z0->mu-mu+ decay mode only


	// Chosen new PDF set; LHAPDF file name conventions.
	//string pdfSet = "cteq61.LHgrid";
	//string pdfSet = "cteq5l.LHgrid";
	string pdfSet = "MRST2001lo.LHgrid";
	//string pdfSet = "MRST2006nnlo.LHgrid";
	pythia.readString("PDF:useLHAPDF = on");
	pythia.readString("PDF:LHAPDFset = " + pdfSet);
	// Allow extrapolation of PDF's beyond x and Q2 boundaries, at own risk.
	// Default behaviour is to freeze PDF's at boundaries.
	pythia.readString("PDF:extrapolateLHAPDF = on");
	// Need to change pT0Ref depending on choice of PDF.
	// One possibility: retune to same <n_charged>.
	//pythia.readString("MultipleInteractions:pT0Ref = 5");



	// Optionally only compare cross sections.
	string partonLevelSwitch = "n";
	string ISRswitch = "n";
	cout << "switch off PartonLevel:all ? (y/n)......";
	cin >> partonLevelSwitch;
	if (partonLevelSwitch == "y") {
		pythia.readString("PartonLevel:all = off");
		//pythia.readString("PartonLevel:ISR = off");
	}
	else if (partonLevelSwitch == "n") {
		cout << "switch off PartonLevel:ISR ? (y/n)......";
		cin >> ISRswitch;
		if (ISRswitch == "y") {
			pythia.readString("PartonLevel:ISR = off");
		}
		else if (ISRswitch == "n") {
			pythia.readString("PartonLevel:ISR = on");
		}
	}


	// phase space cuts
	pythia.readString("PhaseSpace:mHatMin = 20");
	//pythia.readString("PhaseSpace:mHatMax = 200");


	// to tell pythia to initialize for a collision process between two incoming protons
	// (PDG code 2212) at 14000 GeV c.m. energy
	pythia.init( 2212, 2212, 14000.);

	// List changes.
	pythia.settings.listChanged();
	pythia.particleData.listChanged();

	// file
	commonOperations commonOps;
	string dirName = "/";

	//ofstream muFile;
	//string fileName = commonOps.checkANDsetFilepath("MOSESDATA", "/pythia_muons.PYT"); 
	//muFile.open(fileName.c_str());


	string filePath3 = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"pythia_internal_muons.root");
	string treeName = "testTree";
	TFile *treeFile = new TFile(filePath3.c_str(),"recreate");
	TTree *tree = new TTree(treeName.c_str(),treeName.c_str());
	int eventNumber;
	double mHat, thetaHat, cosThetaHat;
	tree->Branch("eventNumber",&eventNumber,"eventNumber/I");
	tree->Branch("mHat",&mHat,"mHat/D");
	tree->Branch("thetaHat",&thetaHat,"thetaHat/D");

	Vec4 p1,p2,pSum,p, pq;
	int idL1=0, idL2=0;

	// run
	int nEvents = 200;

	cout << "\n\n" << "---------------------------Start the run--------------------------" << endl;
	cout << "\n\nenter the number of MUON-PAIRS to loop over (integer>0, default is 200): ";
	cin >> nEvents;
	cout << "start running and booking events to files..." << endl;

	int mod = (int)nEvents/10;
	int evt=1;
	bool passCuts = true;
	while (evt <= nEvents) {
		pythia.next();
	
		eventNumber = evt;
		mHat = pythia.info.mHat();
		//---- in a 2->1->2 process thetaHat is not given in by pythia.info.thetaHat() unlike mHat.------------------
		// therefore we have to calculate it explicitly from the process record (not the event record)
		for (int i=0 ; i<((int)pythia.process.size()-1) ; i++) {
			if (abs(pythia.process[i].id()) == absidLepton && abs(pythia.process[i+1].id()) == absidLepton) {
				idL1 = pythia.process[i].id();
				p1.e(pythia.process[i].e());
				p1.px(pythia.process[i].px());
				p1.py(pythia.process[i].py());
				p1.pz(pythia.process[i].pz());
				idL2 = pythia.process[i+1].id();
	        	        p2.e(pythia.process[i+1].e());
	        	        p2.px(pythia.process[i+1].px());
				p2.py(pythia.process[i+1].py());
				p2.pz(pythia.process[i+1].pz());
			}
		}

		pSum = p1 + p2; // proportional to the boot
		if (idL1 > 0) {
			p1.bstback(pSum); // boost back with beta=-veq{pSum}/pSum^0  (beta = -p/E) 
			p = p1;
		}
		else if (idL2 > 0) {
			p2.bstback(pSum); // boost back with beta=-veq{pSum}/pSum^0  (beta = -p/E)
			p = p2;
		}
		// calculate cos(theta*) relative to the boost
		cosThetaHat = ( p.px()*pSum.px() + p.py()*pSum.py() + p.pz()*pSum.pz() ) /
				(
					sqrt(p.px()*p.px() + p.py()*p.py() + p.pz()*p.pz()) *
					sqrt(pSum.px()*pSum.px() + pSum.py()*pSum.py() + pSum.pz()*pSum.pz())
				); // cosTheta between the vectors wrt the boost direction.
		//cosThetaHat = p.pz()*pq.pz()/(sqrt(p.px()*p.px() + p.py()*p.py() + p.pz()*p.pz())*abs(pq.pz()));
		//-----------------------------------------------------------------------------------------------------------
		thetaHat = acos(cosThetaHat);
		tree->Fill();
		//muFile << evt << "\t" << pythia.info.mHat() << "\t" << pythia.info.thetaHat() << "\n";
		if (evt%mod==0) {cout << "event: " << evt << "\t\t size=" << pythia.event.size() << "  theta=" << thetaHat << endl;}
		if (evt==nEvents) {
			pythia.process.list();
		}
		if (passCuts) { evt++; }
	}
	//muFile.close();
	tree->Print();
	treeFile->Write();
	pythia.statistics();

}

