/************************************************************/
/* class to evaluate the user defined helicity amplitude    */
/* the function here is taken from the "Z LINE SHAPE" paper */
/************************************************************/
#include "helicityAmplitude.h"

using namespace std;

namespace Moses1 {
// this class inherit functions from the "couplings" class

helicityAmplitude::helicityAmplitude(){}
helicityAmplitude::~helicityAmplitude(){}

// to return the regular [qqbar ---> gamma*/Z0 ---> ffbar] helicityAmplitude
dcomplex helicityAmplitude::gethelicityAmplitude(double s, double totalWidth_s,
							double inHelicity, double outHelicity,
							customParticle &inFermion, customParticle &outFermion,
							pdtParticle &propagator, physicalParameters &physPrm) {
	
	dcomplex hAmp (0,0);

	if (propagator.id == PDTGAMMA) { // is photon ?
		hAmp = (inFermion.pdt_prt.charge * outFermion.pdt_prt.charge) / s; // QED amplitude for a photon propagator
	}
	else if (propagator.id == PDTZ) { // is Z0 ?
		hAmp = getCouplings(inHelicity, propagator, inFermion, physPrm) *
			   getCouplings(outHelicity, propagator, outFermion, physPrm) /
			   (s - propagator.mass*propagator.mass + physPrm.i * s * (totalWidth_s/propagator.mass)); // Gideon. PDG
	}
	else {
		cerr << "in helicityAmplitude.cxx: propagator with id=" << propagator.id << " is not supported" << endl;
		exit(-1);
	}
	return(hAmp);
}




// to return the [qqbar ---> Kaluz-Klein gamma/Z0 mode 'n'=0,1,2,... ---> ffbar] helicityAmplitude
dcomplex helicityAmplitude::gethelicityAmplitude(unsigned int nMode, double s, double totalWidth_s,
							double inHelicity, double outHelicity,
							customParticle &inFermion, customParticle &outFermion,
							pdtParticle &propagator, physicalParameters &physPrm) {

	if (propagator.id != PDTGAMMA  &&  propagator.id != PDTZ) {
		cerr << "in helicityAmplitude.cxx: propagator with id=" << propagator.id << " is not supported" << endl;
		exit(-1);
	}
	
	double mass = 0;
	dcomplex hAmp (0,0);

	// for SM propagators
	if (nMode == 0) {hAmp = gethelicityAmplitude(s,totalWidth_s,inHelicity,outHelicity,inFermion,outFermion,propagator,physPrm);}

	// for KK propagators
	else {
		mass = sqrt(propagator.mass*propagator.mass + nMode*physPrm.mStar*nMode*physPrm.mStar);
		hAmp = getCouplings(nMode, inHelicity, propagator, inFermion, physPrm) *
			  getCouplings(nMode, outHelicity, propagator, outFermion, physPrm) /
			  ( s - (mass*mass) + physPrm.i * s * (totalWidth_s/mass) ); // Gideon. PDG
	}
	
	return(hAmp);
}



////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////// Massless Goloun
dcomplex helicityAmplitude::gethelicitySMglounAmplitude(double s, pdtParticle &propagator, physicalParameters &physPrm,customParticle &outFermion1){

        dcomplex zero (1.e-30,0);
        dcomplex hAmp (0,0);

        if (propagator.id != PDTGLU) {
                cerr << "in helicityAmplitude.cxx:gethelicitySMglounAmplitude  propagator with id=" << propagator.id << " is not supported in this function" << endl;
                exit(-1);
        }

        if (s < 4*outFermion1.pdt_prt.mass*outFermion1.pdt_prt.mass){
                        return zero;
                        exit(-1);
                }

        if (propagator.id == PDTGLU) { // is Gloun ?
                hAmp = (physPrm.gQCD * physPrm.gQCD) / s; // QCD amplitude for a Gloun propagator
                //cout << "hAmp SM is " << hAmp << " s is " << s << endl;
                if(abs(hAmp)<abs(zero))
                {
                        return zero;
                }
                else
                {
                        return(hAmp);
                }
        }

        return(zero);
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////// Massive Goloun
dcomplex helicityAmplitude::gethelicityMassiveGAmplitude(unsigned int Lambda_I,unsigned int nMode,
                            double s, double totalWidth_s,
                            double inHelicity1,double inHelicity2,
                            double outHelicity1, double outHelicity2,
                            customParticle &inFermion1, customParticle &inFermion2,
                            customParticle &outFermion1, customParticle &outFermion2,
                                                pdtParticle &propagator, physicalParameters &physPrm,couplings &Gcouplings){

        dcomplex zero (1.e-30,0);
        dcomplex hAmp (0,0);
        dcomplex couplins1 (0,0);
        dcomplex couplins2 (0,0);
        double mass = physPrm.mGluonStar;
        //              getMassiveGlounSchannelCouplings(unsigned int Lambda_I,unsigned int nMode, double helicity1, double helicity2,pdtParticle &propagator,  customParticle &fermion1,physicalParameters &physPrm);
        if(totalWidth_s==0)
        {
                return zero;
        }

	if (propagator.id != PDTGLU) {
                cerr << "in helicityAmplitude.cxx: propagator with id=" << propagator.id << " is not supported in this function" << endl;
                exit(-1);
        }
        else if (s < (outFermion1.pdt_prt.mass+outFermion2.pdt_prt.mass)*(outFermion1.pdt_prt.mass+outFermion2.pdt_prt.mass)){
                        return zero;
                }
        else if (nMode == 0) {hAmp = (physPrm.gQCD * physPrm.gQCD)/ s ;} // QCD amplitude for a Gloun propagator
    else if (nMode == 1){
                //hAmp = (physPrm.gQCD * physPrm.gQCD) / s; // QCD amplitude for a Gloun propagator
                couplins1 = Gcouplings.getMassiveGlounSchannelCouplings(Lambda_I,nMode,inHelicity1, inHelicity2,propagator,inFermion1,inFermion1, physPrm);
                couplins2 = Gcouplings.getMassiveGlounSchannelCouplings(Lambda_I,nMode,outHelicity1,outHelicity2,propagator,outFermion1,outFermion1, physPrm);
                if ((couplins1!=zero) && (couplins2!=zero))
                {
                        // in Couplings g = g * physPrm.g_mGluonStar
                        hAmp = couplins1*couplins2 / ( s - mass * mass + physPrm.i* s * totalWidth_s/mass);
                }
                else
                {
                        return zero;
                }
        }

        else {
                cerr << "in helicityAmplitude.cxx:gethelicityMassiveGAmplitude propagator with id=" << propagator.id << " is not supported" << endl;
                exit(-1);
        }
        if (abs(hAmp)<abs(zero))
        {
                return(zero);
        }
        else
        {
                return(hAmp);
        }

	return zero;

}


}

